from typing import Dict, List, Optional

from .base import Base

class Users(Base):
    @property
    def user_url(self):
        return self.get_url("users")

    def _validate_attributes(self, attributes: Dict[str, str]) -> bool:
        if not attributes:
            return True
        return self._validate_fields(
            set(["ip_address", "user_agent"]), set(attributes.keys())
        )

    def create(
        self,
        email: str = None,
        phone_number: str = None,
        first_name: str = None,
        last_name: str = None,
        middle_name: str = None,
        create_user_as_pending: Optional[bool] = False,
        attributes: Dict[str, str] = {},
    ):
        data = {
            "email": email,
            "phone_number": phone_number,
            "name": {
                "first_name": first_name,
                "middle_name": middle_name,
                "last_name": last_name,
            },
            "create_user_as_pending": create_user_as_pending,
        }
        if attributes and self._validate_attributes(attributes):
            data.update({"attributes": attributes})
        return self._post("{0}".format(self.user_url), data)

    def get(self, user_id: str):
        return self._get("{0}/{1}".format(self.user_url, user_id))

    def get_pending(
        self,
        limit: Optional[int] = None,
        starting_after_id: Optional[str] = None
    ):
        query_params = {}
        if limit:
            query_params.update({"limit": str(limit)})
        if starting_after_id:
            query_params.update({"starting_after_id": starting_after_id})

        return self._get("{0}/{1}".format(self.user_url, "pending"), query_params)

    def delete(self, user_id: str):
        return self._delete("{0}/{1}".format(self.user_url, user_id))

    def update(
        self,
        user_id: str,
        emails: Optional[List[str]] = None,
        phone_numbers: Optional[List[str]] = None,
        first_name: Optional[str] = None,
        middle_name: Optional[str] = None,
        last_name: Optional[str] = None,
        attributes: Optional[Dict[str, str]] = {},
    ):
        data = {}
        name = {}
        if first_name:
            name.update({"first_name": first_name})
        if middle_name:
            name.update({"middle_name": middle_name})
        if last_name:
            name.update({"last_name": last_name})
        if name:
            data.update({"name": name})

        if emails:
            ems = []
            for email in emails:
                ems.append({"email": email})
            data.update({"emails": ems})

        if phone_numbers:
            pns = []
            for phone_number in phone_numbers:
                pns.append({"phone_number": phone_number})
            data.update({"phone_numbers": pns})

        if attributes and self._validate_attributes(attributes):
            data.update({"attributes": attributes})

        return self._put("{0}/{1}".format(self.user_url, user_id), data)

    def delete_email(self, email_id: str):
        return self._delete("{0}/emails/{1}".format(self.user_url, email_id))

    def delete_phone_number(self, phone_id: str):
        return self._delete("{0}/phone_numbers/{1}".format(self.user_url, phone_id))



from typing import Dict, Optional

from .base import _validate_attributes, Base

class OTP(Base):
    def __init__(self, client):
        super().__init__(client)
        self.sms = SMS(client)
        self.whatsapp = Whatsapp(client)

    @property
    def otp_url(self):
        return self.get_url("otps")

    def authenticate(
        self,
        method_id: str,
        code: str,
        attributes: Optional[Dict] = None,
        options: Optional[Dict] = None,
        session_token: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
    ):
        attributes = _validate_attributes(attributes)
        options = self._validate_options(options)
        data = {
            "method_id": method_id,
            "code": code,
        }
        if attributes:
            data["attributes"] = attributes
        if options:
            data["options"] = options
        if session_token:
            data["session_token"] = session_token
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes

        return self._post(
            "{0}/authenticate".format(
                self.otp_url,
            ),
            data=data,
        )

class SMS(Base):
    @property
    def otp_url(self):
        return self.get_url("otps")

    def send(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
    ):
        attributes = _validate_attributes(attributes)
        data = {
            "phone_number": phone_number,
        }
        if expiration_minutes:
            data["expiration_minutes"] = expiration_minutes
        if attributes:
            data["attributes"] = attributes

        return self._post(
            "{0}/sms/send".format(
                self.otp_url,
            ),
            data=data,
        )

    def login_or_create(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        create_user_as_pending: Optional[bool] = False
    ):
        attributes = _validate_attributes(attributes)
        data = {
            "phone_number": phone_number,
            "create_user_as_pending": create_user_as_pending,
        }
        if expiration_minutes:
            data["expiration_minutes"] = expiration_minutes
        if attributes:
            data["attributes"] = attributes

        return self._post(
            "{0}/sms/login_or_create".format(
                self.otp_url,
            ),
            data=data,
        )

class Whatsapp(Base):
    @property
    def otp_url(self):
        return self.get_url("otps")

    def send(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
    ):
        attributes = _validate_attributes(attributes)
        data = {
            "phone_number": phone_number,
        }
        if expiration_minutes:
            data["expiration_minutes"] = expiration_minutes
        if attributes:
            data["attributes"] = attributes

        return self._post(
            "{0}/whatsapp/send".format(
                self.otp_url,
            ),
            data=data,
        )

    def login_or_create(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        create_user_as_pending: Optional[bool] = False
    ):
        attributes = _validate_attributes(attributes)
        data = {
            "phone_number": phone_number,
            "create_user_as_pending": create_user_as_pending,
        }
        if expiration_minutes:
            data["expiration_minutes"] = expiration_minutes
        if attributes:
            data["attributes"] = attributes

        return self._post(
            "{0}/whatsapp/login_or_create".format(
                self.otp_url,
            ),
            data=data,
        )



from typing import Dict, Optional

from .base import _validate_attributes, Base

class MagicLinks(Base):
    def __init__(self, client):
        super().__init__(client)
        self.email = Email(client)

    @property
    def magic_link_url(self):
        return self.get_url("magic_links")

    def authenticate(
        self,
        token: str,
        attributes: Optional[Dict] = None,
        options: Optional[Dict] = None,
        session_token: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
    ):
        attributes = _validate_attributes(attributes)
        options = self._validate_options(options)

        data={
            "token": token,
        }
        if attributes:
            data["attributes"] = attributes
        if options:
            data["options"] = options
        if session_token:
            data["session_token"] = session_token
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes
        return self._post(
            "{0}/authenticate".format(self.magic_link_url),
            data=data,
        )

class Email(Base):
    @property
    def magic_link_url(self):
        return self.get_url("magic_links")

    def send(
        self,
        email: str,
        login_magic_link_url: str,
        signup_magic_link_url: str,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
    ):
        attributes = _validate_attributes(attributes)
        data = {
            "email": email,
            "login_magic_link_url": login_magic_link_url,
            "signup_magic_link_url": signup_magic_link_url,
            "attributes": attributes,
        }
        if login_expiration_minutes:
            data["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes:
            data["signup_expiration_minutes"] = signup_expiration_minutes

        return self._post(
            "{0}/email/send".format(
                self.magic_link_url,
            ),
            data=data,
        )

    def login_or_create(
        self,
        email: str,
        login_magic_link_url: str,
        signup_magic_link_url: str,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        create_user_as_pending: Optional[bool] = False,
    ):
        attributes = _validate_attributes(attributes)
        data = {
           "email": email,
           "login_magic_link_url": login_magic_link_url,
           "signup_magic_link_url": signup_magic_link_url,
           "attributes": attributes,
           "create_user_as_pending": create_user_as_pending,
       }

        if login_expiration_minutes:
            data["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes:
            data["signup_expiration_minutes"] = signup_expiration_minutes

        return self._post(
            "{0}/email/login_or_create".format(
                self.magic_link_url,
            ),
            data=data,
        )

    def invite(
        self,
        email: str,
        invite_magic_link_url: str,
        invite_expiration_minutes: Optional[int] = None,
        attributes: Optional[Dict] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        middle_name: Optional[str] = None,
    ):
        attributes = _validate_attributes(attributes)
        data = {
            "email": email,
            "invite_magic_link_url": invite_magic_link_url,
            "attributes": attributes,
            "name": {
                "first_name": first_name,
                "middle_name": middle_name,
                "last_name": last_name,
            },
        }

        if invite_expiration_minutes:
            data["invite_expiration_minutes"] = invite_expiration_minutes

        return self._post(
            "{0}/email/invite".format(
                self.magic_link_url,
            ),
            data=data,
        )

    def revoke_invite(
        self,
        email: str,
    ):
        return self._post(
            "{0}/email/revoke_invite".format(
                self.magic_link_url,
            ),
            data={
                "email": email,
            },
        )



from typing import Dict, Optional

from .base import _validate_attributes, Base

class Sessions(Base):
    def __init__(self, client):
        super().__init__(client)

    @property
    def sessions_url(self):
        return self.get_url("sessions")

    def get(
        self,
        user_id: str,
    ):
        query_params = {
            "user_id": user_id,
        }
        return self._get(self.sessions_url, query_params)

    def authenticate(
        self,
        session_token: str,
        session_duration_minutes: Optional[int] = None,
    ):
        data = {
            "session_token": session_token,
        }
        if session_duration_minutes:
            data["session_duration_minutes"] = session_duration_minutes

        return self._post(
            "{0}/authenticate".format(self.sessions_url),
            data=data,
        )

    def revoke(
        self,
        session_id: Optional[str] = None,
        session_token: Optional[str] = None,
    ):
        data = {}
        if session_id:
            data["session_id"] = session_id
        if session_token:
            data["session_token"] = session_token

        return self._post(
            "{0}/revoke".format(self.sessions_url),
            data=data,
        )



import warnings

from .api.users import Users
from .api.magic_links import MagicLinks
from .api.otp import OTP
from .api.sessions import Sessions

class Client:
    """
    Stytch API Python client.

    Learn more at https://stytch.com/docs
    """

    def __init__(
        self,
        project_id: str,
        secret: str,
        environment: str,
        suppress_warnings: bool = False,
    ):
        self.project_id = project_id
        self.secret = secret
        self.environment = environment
        self.suppress_warnings = suppress_warnings

        self.users = Users(self)
        self.magic_links = MagicLinks(self)
        self.otps = OTP(self)
        self.sessions = Sessions(self)

    @property
    def base_url(self):
        if self.environment == "test":
            base_url = "https://test.stytch.com/v1/"
            if not self.suppress_warnings:
                warnings.warn("Test version of Stytch not intended for production use")
        elif self.environment == "live":
            base_url = "https://api.stytch.com/v1/"
        else:
            raise Exception("Invalid or missing env. Please specify test or live env")

        return base_url



import pytest
from stytch.client import Client


class TestStytchClient:
    def test_development_env_returns_base_url(self):
        client = Client("project_id", "secret", "test", suppress_warnings=True)
        assert client.base_url == "https://test.stytch.com/v1/"

    def test_production_env_returns_prod_url(self):
        client = Client("project_id", "secret", "live", suppress_warnings=True)
        assert client.base_url == "https://api.stytch.com/v1/"

    def test_no_env_raises_error(self):
        with pytest.raises(Exception):
            _ = Client("project_id", "secret", "invalid env").base_url
            assert _

    def test_Users_controller_exists(self):
        client = Client("project_id", "secret", "development", suppress_warnings=True)
        assert client.Users
        assert client.Users.get
        assert client.Users.update
        assert client.Users.delete
        assert client.Users.create

    def test_MagicLinks_controller_exists(self):
        client = Client("project_id", "secret", "development", suppress_warnings=True)
        assert client.MagicLinks
        assert client.MagicLinks.send
        assert client.MagicLinks.send_by_email
        assert client.MagicLinks.authenticate



import pathlib, re
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

with open('stytch/version.py', 'r') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

# This call to setup() does all the work
setup(
    name="stytch",
    version=version,
    description="Stytch python client",
    long_description=README,
    long_description_content_type="text/markdown",
    download_url="https://github.com/stytchauth/stytch-python",
    keywords=[
        "stytch",
        "user",
        "authentication",
    ],
    author="Stytch",
    author_email="hello@stytch.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["requests>=2.7.0"],
)
